@@ -1,7 +1,13 @@
+import math
 from datetime import datetime, timezone
-from typing import Collection, Dict, List, Mapping, Optional
+from typing import Collection, Dict, List, Mapping, Optional, Tuple
+
+from pydantic import parse_raw_as
 
 from metaphor.common.filter import DatabaseFilter, DatasetFilter
+from metaphor.common.query_history import chunk_query_logs
+from metaphor.common.utils import start_of_day
+from metaphor.snowflake.accessed_object import AccessedObject
 
 try:
     from snowflake.connector.cursor import DictCursor, SnowflakeCursor
@@ -24,6 +30,8 @@ from metaphor.models.metadata_change_event import (
     DatasetStatistics,
     EntityType,
     MaterializationType,
+    QueriedDataset,
+    QueryLog,
     SchemaField,
     SchemaType,
     SourceInfo,
@@ -31,7 +39,14 @@ from metaphor.models.metadata_change_event import (
 )
 from metaphor.snowflake import auth
 from metaphor.snowflake.config import SnowflakeRunConfig
-from metaphor.snowflake.utils import DatasetInfo, SnowflakeTableType
+from metaphor.snowflake.utils import (
+    DatasetInfo,
+    QueryWithParam,
+    SnowflakeTableType,
+    async_execute,
+    exclude_username_clause,
+    fetch_query_history_count,
+)
 
 logger = get_logger(__name__)
 
@@ -55,9 +70,14 @@ class SnowflakeExtractor(BaseExtractor):
         super().__init__(config, "Snowflake metadata crawler", Platform.SNOWFLAKE)
         self._account = config.account
         self._filter = config.filter.normalize().merge(DEFAULT_FILTER)
+        self._query_log_excluded_usernames = config.query_log.excluded_usernames
+        self._query_log_lookback_days = config.query_log.lookback_days
+        self._query_log_fetch_size = config.query_log.fetch_size
+        self._max_concurrency = config.max_concurrency
 
         self._conn = auth.connect(config)
         self._datasets: Dict[str, Dataset] = {}
+        self._logs: List[QueryLog] = []
 
     async def extract(self) -> Collection[ENTITY_TYPES]:
 
@@ -71,9 +91,11 @@ class SnowflakeExtractor(BaseExtractor):
                 if self._filter.includes is None
                 else list(self._filter.includes.keys())
             )
-
             logger.info(f"Databases to include: {databases}")
 
+            shared_databases = self._fetch_shared_databases(cursor)
+            logger.info(f"Shared inbound databases: {shared_databases}")
+
             for database in databases:
                 tables = self._fetch_tables(cursor, database)
                 if len(tables) == 0:
@@ -85,20 +107,30 @@ class SnowflakeExtractor(BaseExtractor):
                 self._fetch_columns(cursor, database)
                 self._fetch_primary_keys(cursor, database)
                 self._fetch_unique_keys(cursor, database)
-                self._fetch_table_info(tables)
+                self._fetch_table_info(tables, database in shared_databases)
 
             self._fetch_tags(cursor)
 
-        logger.debug(self._datasets)
+            if self._query_log_lookback_days > 0:
+                self._fetch_query_logs()
 
-        return self._datasets.values()
+        entities: List[ENTITY_TYPES] = []
+        entities.extend(self._datasets.values())
+        entities.extend(chunk_query_logs(self._logs))
+        return entities
 
     @staticmethod
     def fetch_databases(cursor: SnowflakeCursor) -> List[str]:
         cursor.execute(
             "SELECT database_name FROM information_schema.databases ORDER BY database_name"
         )
-        return [db[0] for db in cursor]
+        return [db[0].lower() for db in cursor]
+
+    @staticmethod
+    def _fetch_shared_databases(cursor: SnowflakeCursor) -> List[str]:
+        cursor.execute("SHOW SHARES")
+        cursor.execute("SELECT * FROM TABLE(RESULT_SCAN(LAST_QUERY_ID()))")
+        return [db[3].lower() for db in cursor if db[1] == "INBOUND"]
 
     FETCH_TABLE_QUERY = """
     SELECT table_schema, table_name, table_type, COMMENT, row_count, bytes
@@ -181,7 +213,7 @@ class SnowflakeExtractor(BaseExtractor):
                 )
             )
 
-    def _fetch_table_info(self, tables: Dict[str, DatasetInfo]) -> None:
+    def _fetch_table_info(self, tables: Dict[str, DatasetInfo], shared: bool) -> None:
         queries, params = [], []
         ddl_tables, updated_time_tables = [], []
         for fullname, table in tables.items():
@@ -193,10 +225,13 @@ class SnowflakeExtractor(BaseExtractor):
                 params.append(fullname)
                 updated_time_tables.append(fullname)
 
-            queries.append(f"get_ddl('table', %s) as \"DDL_{fullname}\"")
-            params.append(fullname)
-            ddl_tables.append(fullname)
+            if not shared:
+                queries.append(f"get_ddl('table', %s) as \"DDL_{fullname}\"")
+                params.append(fullname)
+                ddl_tables.append(fullname)
 
+        if not queries:
+            return
         query = f"SELECT {','.join(queries)}"
 
         cursor = self._conn.cursor(DictCursor)
@@ -310,6 +345,90 @@ class SnowflakeExtractor(BaseExtractor):
                 dataset.schema.tags = []
             dataset.schema.tags.append(self._build_tag_string(key, value))
 
+    def _fetch_query_logs(self) -> None:
+        logger.info("Fetching Snowflake query logs")
+
+        start_date = start_of_day(self._query_log_lookback_days)
+        end_date = start_of_day()
+
+        count = fetch_query_history_count(
+            self._conn, start_date, self._query_log_excluded_usernames, end_date
+        )
+        batches = math.ceil(count / self._query_log_fetch_size)
+        logger.info(f"Total {count} queries, dividing into {batches} batches")
+
+        queries = {
+            x: QueryWithParam(
+                f"""
+                SELECT q.QUERY_ID, q.USER_NAME, QUERY_TEXT, START_TIME, TOTAL_ELAPSED_TIME,
+                  BYTES_SCANNED, BYTES_WRITTEN, ROWS_PRODUCED, DIRECT_OBJECTS_ACCESSED, OBJECTS_MODIFIED
+                FROM SNOWFLAKE.ACCOUNT_USAGE.QUERY_HISTORY q
+                JOIN SNOWFLAKE.ACCOUNT_USAGE.ACCESS_HISTORY a
+                  ON a.QUERY_ID = q.QUERY_ID
+                WHERE EXECUTION_STATUS = 'SUCCESS'
+                  AND START_TIME > %s AND START_TIME <= %s
+                  AND QUERY_START_TIME > %s AND QUERY_START_TIME <= %s
+                  {exclude_username_clause(self._query_log_excluded_usernames)}
+                ORDER BY q.QUERY_ID
+                LIMIT {self._query_log_fetch_size} OFFSET %s
+                """,
+                (
+                    start_date,
+                    end_date,
+                    start_date,
+                    end_date,
+                    *self._query_log_excluded_usernames,
+                    x * self._query_log_fetch_size,
+                ),
+            )
+            for x in range(batches)
+        }
+        async_execute(
+            self._conn,
+            queries,
+            "fetch_query_logs",
+            self._max_concurrency,
+            self._parse_access_logs,
+        )
+
+        logger.info(f"Fetched {len(self._logs)} query logs")
+
+    def _parse_access_logs(self, batch_number: str, access_logs: List[Tuple]) -> None:
+        logger.info(f"access logs batch #{batch_number}")
+        for (
+            query_id,
+            username,
+            query_text,
+            start_time,
+            elapsed_time,
+            bytes_scanned,
+            bytes_written,
+            rows_produced,
+            accessed_objects,
+            modified_objects,
+        ) in access_logs:
+            try:
+                sources = self._parse_accessed_objects(accessed_objects)
+                targets = self._parse_accessed_objects(modified_objects)
+
+                query_log = QueryLog(
+                    id=f"{str(DataPlatform.SNOWFLAKE)}:{query_id}",
+                    platform=DataPlatform.SNOWFLAKE,
+                    start_time=start_time,
+                    duration=float(elapsed_time / 1000.0),
+                    user_id=username,
+                    rows_written=float(rows_produced) if rows_produced else None,
+                    bytes_read=float(bytes_scanned) if bytes_scanned else None,
+                    bytes_written=float(bytes_written) if bytes_written else None,
+                    sources=sources,
+                    targets=targets,
+                    sql=query_text,
+                )
+
+                self._logs.append(query_log)
+            except Exception:
+                logger.exception(f"access log processing error, query id: {query_id}")
+
     @staticmethod
     def _build_tag_string(tag_key: str, tag_value: str) -> str:
         return f"{tag_key}={tag_value}"
@@ -360,3 +479,34 @@ class SnowflakeExtractor(BaseExtractor):
             f"https://{account}.snowflakecomputing.com/console#/data/tables/detail?"
             f"databaseName={db}&schemaName={schema}&tableName={table}"
         )
+
+    @staticmethod
+    def _parse_accessed_objects(raw_objects: str) -> List[QueriedDataset]:
+        objects = parse_raw_as(List[AccessedObject], raw_objects)
+        queried_datasets: List[QueriedDataset] = []
+        for obj in objects:
+            if not obj.objectDomain or obj.objectDomain.upper() not in (
+                "TABLE",
+                "VIEW",
+                "MATERIALIZED VIEW",
+            ):
+                continue
+
+            table_name = obj.objectName.lower()
+            parts = table_name.split(".")
+            if len(parts) != 3:
+                logger.debug(f"Invalid table name {table_name}, skip")
+                continue
+
+            db, schema, table = parts
+
+            queried_datasets.append(
+                QueriedDataset(
+                    database=db,
+                    schema=schema,
+                    table=table,
+                    columns=[col.columnName for col in obj.columns] or None,
+                )
+            )
+
+        return queried_datasets
