@@ -66,6 +66,7 @@ def mapping_data_object_type(type_: SourceType) -> ThoughtSpotDataObjectType:
     return mapping.get(type_)
 
 
+# https://docs.thoughtspot.com/software/latest/chart-types
 def mapping_chart_type(type_: str) -> Optional[ChartType]:
     mapping = {
         "COLUMN": ChartType.COLUMN,
@@ -170,7 +171,7 @@ class ThoughtSpot:
         return _client(result.token)
 
     @staticmethod
-    def fetch_connections(client: RestapisdkClient) -> List[Metadata]:
+    def fetch_connections(client: RestapisdkClient) -> List[ConnectionMetadata]:
         supported_platform = set(c.value for c in ConnectionType)
 
         headers: List[ConnectionHeader] = ThoughtSpot._fetch_headers(
@@ -197,7 +198,7 @@ class ThoughtSpot:
         logger.info(f"{mtype} ids: {ids}")
 
         obj = ThoughtSpot._fetch_object_detail(client.metadata, ids, detail_type)
-        logger.debug(f"fetch_object_detail: {obj}")
+        logger.debug(f"[{mtype}] fetch_object_detail: {json.dumps(obj)}")
 
         # Because mypy can't handle dynamic type variables properly, we skip type-checking here.
         return parse_obj_as(List[target_type], obj)  # type: ignore
